<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * This script handles the report over sessions generation in PDF for a single group.
 * It may produce a group pdf report.
 * groupid must be provided.
 * This script should be sheduled in a redirect bouncing process for maintaining
 * memory level available for huge batches.
 *
 * @package    report_trainingsessions
 * @author     Valery Fremaux (valery.fremaux@gmail.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require('../../../../config.php');
require_once($CFG->dirroot.'/blocks/use_stats/locallib.php');
require_once($CFG->dirroot.'/course/report/trainingsessions/locallib.php');
require_once($CFG->libdir.'/excellib.class.php');

$id = required_param('id', PARAM_INT); // The course id.
$groupid = required_param('groupid', PARAM_INT); // The group id.

ini_set('memory_limit', '512M');

if (!$course = $DB->get_record('course', array('id' => $id))) {
    // Do NOT print_error here as we are a document writer.
    die ('Invalid course ID');
}
$context = context_course::instance($course->id);

$input = report_trainingsessions_batch_input($course);

// Security.

trainingsessions_back_office_access($course);

// Compute target group.

$group = $DB->get_record('groups', array('id' => $groupid));

if ($groupid) {
    $targetusers = groups_get_members($groupid);
} else {
    $targetusers = get_enrolled_users($context);
}

// Filter out non compiling users.
trainingsessions_filter_unwanted_users($targetusers);

// Print result.
$result = new StdClass();
$result->groupname = $group->name;
$result->from = $input->from;
$result->to = $input->to;
$result->groupid = $groupid;

if (!empty($targetusers)) {
    foreach ($targetusers as $auser) {
        $result->members[$auser->id] = new StdClass;
        $result->members[$auser->id]->userid = $auser->id;
        $result->members[$auser->id]->firstname = $auser->firstname;
        $result->members[$auser->id]->lastname = $auser->lastname;
        $result->members[$auser->id]->idnumber = $auser->idnumber;
        report_trainingsessions_get_usersessions($result->members[$auser->id], $auser->id);
    }
    // Sending HTTP headers
    ob_end_clean();

    echo json_encode($result);
}
exit(0);
// echo '200';